forceRequiredAddOn("Weapon_Bow");

datablock ProjectileData(HL1BoltProjectile : arrowProjectile)
{
	projectileShapeName = "./shapes/crossbow/crossbow_bolt.dts";
	directDamage        = $Pref::HL1Guns_DMGCrossbow;

	bounceAngle         = 180; 
	muzzleVelocity      = 200;
	uiName = "HL1 Bolt";
};
function HL1BoltProjectile::onCollision(%this, %obj, %col, %fade, %pos, %normal, %velocity)
{
	Parent::onCollision(%this, %obj, %col, %fade, %pos, %normal, %velocity);
	
	%reflectVec = getBounceVector(%velocity, %normal);
	%dot = vectorDot(%velocity, %reflectVec);
	%dot = %dot / (vectorLen(%velocity) * vectorLen(%reflectVec));
	%acos = mACos(%dot);
	%angle = mRadtoDeg(%acos);
	
	//// vars
	%ricoMinimumAngle = 85; 	// minimum angle that can ricochet
	%ricoMinimumFactor = 0.5;
	
	if(%angle > %ricoMinimumAngle)
		return;
		
	%chance = getRandom() * 100;
	
	if(%chance > 10)
	{
		%bounceFac = getRandom();
		if(%bounceFac < %ricoMinimumFactor)
			%bounceFac = %ricoMinimumFactor;
		
		//// sound
		serverplay3d("HL1CrossbowRic" @ getRandom(1, 2) @ "Sound", %obj.getPosition() SPC "0 0 0 0");
		
		%obj.bounce(%bounceFac, %client);
	}
}

datablock ItemData(HL1CrossbowItem)
{
	category = "Weapon";  // Mission editor category
	className = "Weapon"; // For inventory system

	 // Basic Item Properties
	shapeFile = "./shapes/crossbow/crossbow.dts";
	rotate = false;
	mass = 1;
	density = 0.2;
	elasticity = 0.2;
	friction = 0.6;
	emap = true;

	//gui stuff
	uiName = "HL1 Crossbow";
	iconName = "./icons/crossbow";
	doColorShift = true;
	colorShiftColor = "0.25 0.25 0.25 1.000";

	 // Dynamic properties defined by the scripts
	image = HL1CrossbowImage;
	canDrop = true;
	
	ghostToReload = true;
};

////////////////
//weapon image//
////////////////
datablock ShapeBaseImageData(HL1CrossbowImage)
{
   // Basic Item properties
	shapeFile = "./shapes/crossbow/crossbow.dts";
   emap = true;

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 0;
   offset = "0 0 0";
   eyeOffset = 0; //"0.7 1.2 -0.5";
   rotation = eulerToMatrix( "0 0 0" );

   // When firing from a point offset from the eye, muzzle correction
   // will adjust the muzzle vector to point to the eye LOS point.
   // Since this weapon doesn't actually fire from the muzzle point,
   // we need to turn this off.  
   correctMuzzleVector = true;

   // Add the WeaponImage namespace as a parent, WeaponImage namespace
   // provides some hooks into the inventory system.
   className = "WeaponImage";

   // Projectile && Ammo.
   item = HL1CrossbowItem;
   ammo = " ";
   projectile = HL1BoltProjectile;
   projectileType = Projectile;

	casing = gunShellDebris;
	shellExitDir        = "1.0 -1.3 1.0";
	shellExitOffset     = "0 0 0";
	shellExitVariance   = 15.0;	
	shellVelocity       = 7.0;

   //melee particles shoot from eye node for consistancy
   melee = false;
   //raise your arm up or not
   armReady = true;

   doColorShift = true;
   colorShiftColor = HL1CrossbowItem.colorShiftColor;//"0.400 0.196 0 1.000";

   //casing = " ";

   // Images have a state system which controls how the animations
   // are run, which sounds are played, script callbacks, etc. This
   // state system is downloaded to the client so that clients can
   // predict state changes and animate accordingly.  The following
   // system supports basic ready->fire->reload transitions as
   // well as a no-ammo->dryfire idle state.

   // Initial start up state
	stateName[0]                     	= "Activate";
	stateTimeoutValue[0]             	= 0.15;
	stateTransitionOnTimeout[0]       	= "Ready";
	stateSound[0]						= weaponSwitchSound;

	// Ready for input
	stateName[1]                     	= "Ready";
	stateTransitionOnTriggerDown[1]  	= "Fire";
	stateTransitionOnNotLoaded[1]		= "FireBolt";
	stateTransitionOnNoAmmo[1]			= "ImageReload";
	stateAllowImageChange[1]         	= true;
	stateSequence[1]					= "Ready";

	// Explosive Shot
	stateName[6]                    	= "FireBolt";
	stateTransitionOnTimeout[6]     	= "Smoke";
	stateTimeoutValue[6]            	= 1.25;
	stateFire[6]                    	= true;
	stateAllowImageChange[6]        	= false;
	stateSequence[6]                	= "Fire";
	stateScript[6]                  	= "onExplosiveFire";
	stateWaitForTimeout[6]				= true;
	stateSound[6]						= HL1CrossbowFireSound;
	
	// Normal fire
	stateName[2]                    	= "Fire";
	stateTransitionOnTimeout[2]     	= "Smoke";
	stateTimeoutValue[2]            	= 0.75; // 80RPM
	stateFire[2]                    	= true;
	stateAllowImageChange[2]        	= false;
	stateSequence[2]                	= "Fire";
	stateScript[2]                  	= "onFire";
	stateWaitForTimeout[2]				= true;
	stateSound[2]						= HL1CrossbowFireSound;

	// Smoke and wait
	stateName[3] 						= "Smoke";
	stateTimeoutValue[3]            	= 0.01;
	stateTransitionOnTriggerUp[3]     	= "Ready";
	stateTransitionOnNoAmmo[3]			= "ImageReload";
	
	// Reload effects
	// Mag out
	stateName[11]						= "ImageReload";
	stateScript[11]						= "IRMagout";
	stateSequence[11]					= "Disengaged";
	stateTimeoutValue[11]				= 0.5;
	stateTransitionOnTimeout[11]		= "IR-Magin";
	stateSound[11]						= brickMoveSound;
	
	// Mag in
	stateName[12]						= "IR-Magin";
	stateScript[12]						= "IRMagin";
	stateSequence[12]					= "PostFire";
	stateTimeoutValue[12]				= 0.5;
	stateTransitionOnTimeout[12]		= "IR-Chamber";
	stateSound[12]						= brickMoveSound;
	
	// Chamber
	stateName[13]						= "IR-Chamber";
	stateScript[13]						= "IRChamber";
	stateSequence[13]					= "Fire";
	stateTimeoutValue[13]				= 0.2;
	stateSequence[13]					= "Fire";
	stateTransitionOnTimeout[13]		= "Ready";
	stateSound[13]						= Hl1CrossbowChargeSound;
};

function HL1CrossbowImage::IRMagout(%this, %player, %slot)
{
	%player.playThread(2, shiftTo);
}
function HL1CrossbowImage::IRMagin(%this, %player, %slot)
{
	%player.playThread(2, shiftAway);
}
function HL1CrossbowImage::IRChamber(%this, %player, %slot)
{
	%player.HL1CrossbowRounds = 5;
	%player.playThread(2, plant);
	%player.setImageAmmo(0, 1);
}

function HL1CrossbowImage::onMount(%this, %player, %slot)
{
	Parent::onMount(%this, %player, %slot);
	if(%player.HL1CrossbowRounds <= 0)
	{
		%player.setImageAmmo(0, 0);
	}
}

function HL1CrossbowImage::onFire(%this, %player, %slot)
{	
	if(%player.getDamagePercent() >= 1)
		return;

	%player.playThread(2, jump);
	
	// small explanation of what i'm doing here
	// first we find what we're actually hitting, then we find a place that's right before we hit it on the same vector
	// and spawn the projectile there. projectile hits instantly, but we preserve all projectile functionality

	// the distance the projectile will instantly travel
	%flatness = 256;
	
	// how far backwards behind the target the projectile spawns
	%engageDistance = 1;
	
	// raycast
	%start = %player.getMuzzlePoint(%slot);
	%end = vectorScale(%player.getMuzzleVector(%slot), %flatness);
	%mask = $TypeMasks::PlayerObjectType | 
		$TypeMasks::VehicleObjectType | 
		$TypeMasks::fxBrickObjectType | 
		$TypeMasks::StaticObjectType | 
		$TypeMasks::InteriorObjectType | 
		$TypeMasks::TerrainObjectType | 
		$TypeMasks::ForceFieldObjectType;
	
	%target = containerRayCast(%start, vectorAdd(%start, %end), %mask);

	if(!%target)
	{
		%hitPosition = vectorAdd(%start, %end);
	}
	else
	{
		%hitPosition = getWords(%target, 1, 3);
	}
	
	%backwardsVelocity = vectorScale(%player.getMuzzleVector(%slot), -1);
	%backwardsPoint = vectorAdd(%hitPosition, vectorScale(%backwardsVelocity, %engageDistance));
	
	%projectile = %this.projectile;
	%spread = 0.0;
	%shellcount = 1;
	
	%client = %player.client;
	%pos = %player.getHackPosition();

	for(%shell=0; %shell<%shellcount; %shell++)
	{
		%vector = %player.getMuzzleVector(%slot);
		%playerectVelocity = %player.getVelocity();
		%vector1 = VectorScale(%vector, %projectile.muzzleVelocity);
		%vector2 = VectorScale(%playerectVelocity, %projectile.velInheritFactor);
		%velocity = VectorAdd(%vector1,%vector2);
		%x = (getRandom() - 0.5) * 10 * 3.1415926 * %spread;
		%y = (getRandom() - 0.5) * 10 * 3.1415926 * %spread;
		%z = (getRandom() - 0.5) * 10 * 3.1415926 * %spread;
		%mat = MatrixCreateFromEuler(%x @ " " @ %y @ " " @ %z);
		%velocity = MatrixMulVector(%mat, %velocity);
		
		%p = new (%this.projectileType)()
		{
			dataBlock = %projectile;
			initialVelocity = %velocity;
			initialPosition = %backwardsPoint;
			sourceObject = %player;
			sourceSlot = %slot;
			client = %player.client;
		};
		MissionCleanup.add(%p);
		%dmg = 1 * $Pref::HalfLife1_DamageMult;
		%p.setScale(%dmg SPC %dmg SPC %dmg);
	}
	
	%player.HL1CrossbowRounds--;
	if(%player.HL1CrossbowRounds <= 0)
	{
		%player.setImageAmmo(0, 0);
	}
	
	return %p;
}

function HL1CrossbowImage::onExplosiveFire(%this, %player, %slot)
{	

	if(%player.getDamagePercent() >= 1)
		return;

	%player.playThread(2, jump);

	%projectile = tankShellProjectile;
	%spread = 0.000225;
	%shellcount = 1;
	
	if(%player.HL1CrossbowRounds <= 0)
	{
		%shellcount = 0;
	}

	
	%client = %player.client;
	%pos = %player.getHackPosition();

	for(%shell=0; %shell<%shellcount; %shell++)
	{
		%vector = %player.getMuzzleVector(%slot);
		%playerectVelocity = %player.getVelocity();
		%vector1 = VectorScale(%vector, 100);
		%vector2 = VectorScale(%playerectVelocity, %projectile.velInheritFactor);
		%velocity = VectorAdd(%vector1,%vector2);
		%x = (getRandom() - 0.5) * 10 * 3.1415926 * %spread;
		%y = (getRandom() - 0.5) * 10 * 3.1415926 * %spread;
		%z = (getRandom() - 0.5) * 10 * 3.1415926 * %spread;
		%mat = MatrixCreateFromEuler(%x @ " " @ %y @ " " @ %z);
		%velocity = MatrixMulVector(%mat, %velocity);

		%p = new (%this.projectileType)()
		{
			dataBlock = %projectile;
			initialVelocity = %velocity;
			initialPosition = %player.getMuzzlePoint(%slot);
			sourceObject = %player;
			sourceSlot = %slot;
			client = %player.client;
		};
		MissionCleanup.add(%p);
		%dmg = 1 * 0.25;
		%p.setScale(%dmg SPC %dmg SPC %dmg);
	}
	
	%player.HL1CrossbowRounds--;
	if(%player.HL1CrossbowRounds <= 0)
	{
		%player.setImageAmmo(0, 0);
	}
	
	return %p;
}

function HL1CrossbowImage::HandleReload(%this, %player, %slot)
{
	if(%player.HL1CrossbowRounds >= 5 || %player.getImageState(0) !$= "Ready")
		return false;
	else
	{
		%player.setImageAmmo(0, 0);
		return true;
	}
}